from blueprints.structural_sections.steel.profile_definitions.chs_profile import CHSProfile
from blueprints.type_alias import MM

class CHS:
    CHS21_3x2_3: CHSProfile
    CHS21_3x2_6: CHSProfile
    CHS21_3x3_2: CHSProfile
    CHS26_9x2_3: CHSProfile
    CHS26_9x2_6: CHSProfile
    CHS26_9x3_2: CHSProfile
    CHS33_7x2_6: CHSProfile
    CHS33_7x3_2: CHSProfile
    CHS33_7x4: CHSProfile
    CHS42_4x2_6: CHSProfile
    CHS42_4x3_2: CHSProfile
    CHS42_4x4: CHSProfile
    CHS48_3x2_6: CHSProfile
    CHS48_3x3_2: CHSProfile
    CHS48_3x4: CHSProfile
    CHS48_3x5: CHSProfile
    CHS60_3x2_6: CHSProfile
    CHS60_3x3_2: CHSProfile
    CHS60_3x4: CHSProfile
    CHS60_3x5: CHSProfile
    CHS76_1x2_6: CHSProfile
    CHS76_1x3_2: CHSProfile
    CHS76_1x4: CHSProfile
    CHS76_1x5: CHSProfile
    CHS88_9x3_2: CHSProfile
    CHS88_9x4: CHSProfile
    CHS88_9x5: CHSProfile
    CHS88_9x6_3: CHSProfile
    CHS101_6x3_2: CHSProfile
    CHS101_6x4: CHSProfile
    CHS101_6x5: CHSProfile
    CHS101_6x6_3: CHSProfile
    CHS101_6x8: CHSProfile
    CHS101_6x10: CHSProfile
    CHS114_3x3_2: CHSProfile
    CHS114_3x4: CHSProfile
    CHS114_3x5: CHSProfile
    CHS114_3x6_3: CHSProfile
    CHS114_3x8: CHSProfile
    CHS114_3x10: CHSProfile
    CHS139_7x4: CHSProfile
    CHS139_7x5: CHSProfile
    CHS139_7x6_3: CHSProfile
    CHS139_7x8: CHSProfile
    CHS139_7x10: CHSProfile
    CHS139_7x12_5: CHSProfile
    CHS168_3x4: CHSProfile
    CHS168_3x5: CHSProfile
    CHS168_3x6_3: CHSProfile
    CHS168_3x8: CHSProfile
    CHS168_3x10: CHSProfile
    CHS168_3x12_5: CHSProfile
    CHS177_8x5: CHSProfile
    CHS177_8x6_3: CHSProfile
    CHS177_8x8: CHSProfile
    CHS177_8x10: CHSProfile
    CHS177_8x12_5: CHSProfile
    CHS193_7x5: CHSProfile
    CHS193_7x6_3: CHSProfile
    CHS193_7x8: CHSProfile
    CHS193_7x10: CHSProfile
    CHS193_7x12_5: CHSProfile
    CHS193_7x14_2: CHSProfile
    CHS193_7x16: CHSProfile
    CHS219_1x5: CHSProfile
    CHS219_1x6_3: CHSProfile
    CHS219_1x8: CHSProfile
    CHS219_1x10: CHSProfile
    CHS219_1x12_5: CHSProfile
    CHS219_1x14_2: CHSProfile
    CHS219_1x16: CHSProfile
    CHS219_1x20: CHSProfile
    CHS244_5x5: CHSProfile
    CHS244_5x6_3: CHSProfile
    CHS244_5x8: CHSProfile
    CHS244_5x10: CHSProfile
    CHS244_5x12_5: CHSProfile
    CHS244_5x14_2: CHSProfile
    CHS244_5x16: CHSProfile
    CHS244_5x20: CHSProfile
    CHS244_5x25: CHSProfile
    CHS273x5: CHSProfile
    CHS273x6_3: CHSProfile
    CHS273x8: CHSProfile
    CHS273x10: CHSProfile
    CHS273x12_5: CHSProfile
    CHS273x14_2: CHSProfile
    CHS273x16: CHSProfile
    CHS273x20: CHSProfile
    CHS273x25: CHSProfile
    CHS323_9x5: CHSProfile
    CHS323_9x6_3: CHSProfile
    CHS323_9x8: CHSProfile
    CHS323_9x10: CHSProfile
    CHS323_9x12_5: CHSProfile
    CHS323_9x14_2: CHSProfile
    CHS323_9x16: CHSProfile
    CHS323_9x20: CHSProfile
    CHS323_9x25: CHSProfile
    CHS355_6x6_3: CHSProfile
    CHS355_6x8: CHSProfile
    CHS355_6x10: CHSProfile
    CHS355_6x12_5: CHSProfile
    CHS355_6x14_2: CHSProfile
    CHS355_6x16: CHSProfile
    CHS355_6x20: CHSProfile
    CHS355_6x25: CHSProfile
    CHS406_4x6_3: CHSProfile
    CHS406_4x8: CHSProfile
    CHS406_4x10: CHSProfile
    CHS406_4x12_5: CHSProfile
    CHS406_4x14_2: CHSProfile
    CHS406_4x16: CHSProfile
    CHS406_4x20: CHSProfile
    CHS406_4x25: CHSProfile
    CHS406_4x30: CHSProfile
    CHS406_4x40: CHSProfile
    CHS457x6_3: CHSProfile
    CHS457x8: CHSProfile
    CHS457x10: CHSProfile
    CHS457x12_5: CHSProfile
    CHS457x14_2: CHSProfile
    CHS457x16: CHSProfile
    CHS457x20: CHSProfile
    CHS457x25: CHSProfile
    CHS457x30: CHSProfile
    CHS457x40: CHSProfile
    CHS508x6_3: CHSProfile
    CHS508x8: CHSProfile
    CHS508x10: CHSProfile
    CHS508x12_5: CHSProfile
    CHS508x14_2: CHSProfile
    CHS508x16: CHSProfile
    CHS508x20: CHSProfile
    CHS508x25: CHSProfile
    CHS508x30: CHSProfile
    CHS508x40: CHSProfile
    CHS610x6_3: CHSProfile
    CHS610x8: CHSProfile
    CHS610x10: CHSProfile
    CHS610x12_5: CHSProfile
    CHS610x14_2: CHSProfile
    CHS610x16: CHSProfile
    CHS610x20: CHSProfile
    CHS610x25: CHSProfile
    CHS610x30: CHSProfile
    CHS610x40: CHSProfile
    CHS711x6_3: CHSProfile
    CHS711x8: CHSProfile
    CHS711x10: CHSProfile
    CHS711x12_5: CHSProfile
    CHS711x14_2: CHSProfile
    CHS711x16: CHSProfile
    CHS711x20: CHSProfile
    CHS711x25: CHSProfile
    CHS711x30: CHSProfile
    CHS711x40: CHSProfile
    CHS762x6_3: CHSProfile
    CHS762x8: CHSProfile
    CHS762x10: CHSProfile
    CHS762x12_5: CHSProfile
    CHS762x14_2: CHSProfile
    CHS762x16: CHSProfile
    CHS762x20: CHSProfile
    CHS762x25: CHSProfile
    CHS762x30: CHSProfile
    CHS762x40: CHSProfile
    CHS813x8: CHSProfile
    CHS813x10: CHSProfile
    CHS813x12_5: CHSProfile
    CHS813x14_2: CHSProfile
    CHS813x16: CHSProfile
    CHS813x20: CHSProfile
    CHS813x25: CHSProfile
    CHS813x30: CHSProfile
    CHS914x8: CHSProfile
    CHS914x10: CHSProfile
    CHS914x12_5: CHSProfile
    CHS914x14_2: CHSProfile
    CHS914x16: CHSProfile
    CHS914x20: CHSProfile
    CHS914x25: CHSProfile
    CHS914x30: CHSProfile
    CHS1016x8: CHSProfile
    CHS1016x10: CHSProfile
    CHS1016x12_5: CHSProfile
    CHS1016x14_2: CHSProfile
    CHS1016x16: CHSProfile
    CHS1016x20: CHSProfile
    CHS1016x25: CHSProfile
    CHS1016x30: CHSProfile
    CHS1067x10: CHSProfile
    CHS1067x12_5: CHSProfile
    CHS1067x14_2: CHSProfile
    CHS1067x16: CHSProfile
    CHS1067x20: CHSProfile
    CHS1067x25: CHSProfile
    CHS1067x30: CHSProfile
    CHS1168x10: CHSProfile
    CHS1168x12_5: CHSProfile
    CHS1168x14_2: CHSProfile
    CHS1168x16: CHSProfile
    CHS1168x20: CHSProfile
    CHS1168x25: CHSProfile
    CHS1219x10: CHSProfile
    CHS1219x12_5: CHSProfile
    CHS1219x14_2: CHSProfile
    CHS1219x16: CHSProfile
    CHS1219x20: CHSProfile
    CHS1219x25: CHSProfile
    CHS1219x30: CHSProfile
    CHS1219x32: CHSProfile
    CHS1219x36: CHSProfile
    CHS1219x40: CHSProfile
    CHS1420x10: CHSProfile
    CHS1420x12_5: CHSProfile
    CHS1420x14_2: CHSProfile
    CHS1420x16: CHSProfile
    CHS1420x20: CHSProfile
    CHS1420x25: CHSProfile
    CHS1420x30: CHSProfile
    CHS1420x32: CHSProfile
    CHS1420x36: CHSProfile
    CHS1420x40: CHSProfile
    CHS1620x10: CHSProfile
    CHS1620x12_5: CHSProfile
    CHS1620x14_2: CHSProfile
    CHS1620x16: CHSProfile
    CHS1620x20: CHSProfile
    CHS1620x25: CHSProfile
    CHS1620x30: CHSProfile
    CHS1620x32: CHSProfile
    CHS1620x36: CHSProfile
    CHS1620x40: CHSProfile
    CHS1820x12_5: CHSProfile
    CHS1820x14_2: CHSProfile
    CHS1820x16: CHSProfile
    CHS1820x20: CHSProfile
    CHS1820x25: CHSProfile
    CHS1820x30: CHSProfile
    CHS1820x32: CHSProfile
    CHS1820x36: CHSProfile
    CHS1820x40: CHSProfile
    CHS2020x14_2: CHSProfile
    CHS2020x16: CHSProfile
    CHS2020x20: CHSProfile
    CHS2020x25: CHSProfile
    CHS2020x30: CHSProfile
    CHS2020x32: CHSProfile
    CHS2020x36: CHSProfile
    CHS2020x40: CHSProfile
    CHS2220x14_2: CHSProfile
    CHS2220x16: CHSProfile
    CHS2220x20: CHSProfile
    CHS2220x25: CHSProfile
    CHS2220x30: CHSProfile
    CHS2220x32: CHSProfile
    CHS2220x36: CHSProfile
    CHS2220x40: CHSProfile

class __CHSProfileParameters:
    name: str
    outer_diameter: MM
    wall_thickness: MM

CHS_PROFILES_DATABASE: dict[str, __CHSProfileParameters]
