from blueprints.structural_sections.steel.profile_definitions.rhs_profile import RHSProfile
from blueprints.type_alias import MM

class RHS:
    RHS50x30x2_6: RHSProfile
    RHS50x30x3_2: RHSProfile
    RHS50x30x4: RHSProfile
    RHS50x30x5: RHSProfile
    RHS60x40x2_6: RHSProfile
    RHS60x40x3_2: RHSProfile
    RHS60x40x4: RHSProfile
    RHS60x40x5: RHSProfile
    RHS60x40x6_3: RHSProfile
    RHS80x40x3_2: RHSProfile
    RHS80x40x4: RHSProfile
    RHS80x40x5: RHSProfile
    RHS80x40x6_3: RHSProfile
    RHS80x40x8: RHSProfile
    RHS90x50x3_2: RHSProfile
    RHS90x50x4: RHSProfile
    RHS90x50x5: RHSProfile
    RHS90x50x6_3: RHSProfile
    RHS90x50x8: RHSProfile
    RHS100x50x3_2: RHSProfile
    RHS100x50x4: RHSProfile
    RHS100x50x5: RHSProfile
    RHS100x50x6_3: RHSProfile
    RHS100x50x8: RHSProfile
    RHS100x60x3_2: RHSProfile
    RHS100x60x4: RHSProfile
    RHS100x60x5: RHSProfile
    RHS100x60x6_3: RHSProfile
    RHS100x60x8: RHSProfile
    RHS120x60x4: RHSProfile
    RHS120x60x5: RHSProfile
    RHS120x60x6_3: RHSProfile
    RHS120x60x8: RHSProfile
    RHS120x60x10: RHSProfile
    RHS120x80x4: RHSProfile
    RHS120x80x5: RHSProfile
    RHS120x80x6_3: RHSProfile
    RHS120x80x8: RHSProfile
    RHS120x80x10: RHSProfile
    RHS140x80x4: RHSProfile
    RHS140x80x5: RHSProfile
    RHS140x80x6_3: RHSProfile
    RHS140x80x8: RHSProfile
    RHS140x80x10: RHSProfile
    RHS150x100x4: RHSProfile
    RHS150x100x5: RHSProfile
    RHS150x100x6_3: RHSProfile
    RHS150x100x8: RHSProfile
    RHS150x100x10: RHSProfile
    RHS150x100x12_5: RHSProfile
    RHS160x80x4: RHSProfile
    RHS160x80x5: RHSProfile
    RHS160x80x6_3: RHSProfile
    RHS160x80x8: RHSProfile
    RHS160x80x10: RHSProfile
    RHS160x80x12_5: RHSProfile
    RHS180x100x4: RHSProfile
    RHS180x100x5: RHSProfile
    RHS180x100x6_3: RHSProfile
    RHS180x100x8: RHSProfile
    RHS180x100x10: RHSProfile
    RHS180x100x12_5: RHSProfile
    RHS200x100x4: RHSProfile
    RHS200x100x5: RHSProfile
    RHS200x100x6_3: RHSProfile
    RHS200x100x8: RHSProfile
    RHS200x100x10: RHSProfile
    RHS200x100x12_5: RHSProfile
    RHS200x100x16: RHSProfile
    RHS200x120x6_3: RHSProfile
    RHS200x120x8: RHSProfile
    RHS200x120x10: RHSProfile
    RHS200x120x12_5: RHSProfile
    RHS250x150x6_3: RHSProfile
    RHS250x150x8: RHSProfile
    RHS250x150x10: RHSProfile
    RHS250x150x12_5: RHSProfile
    RHS250x150x14_2: RHSProfile
    RHS250x150x16: RHSProfile
    RHS260x180x6_3: RHSProfile
    RHS260x180x8: RHSProfile
    RHS260x180x10: RHSProfile
    RHS260x180x12_5: RHSProfile
    RHS260x180x14_2: RHSProfile
    RHS260x180x16: RHSProfile
    RHS300x200x6_3: RHSProfile
    RHS300x200x8: RHSProfile
    RHS300x200x10: RHSProfile
    RHS300x200x12_5: RHSProfile
    RHS300x200x14_2: RHSProfile
    RHS300x200x16: RHSProfile
    RHS350x250x6_3: RHSProfile
    RHS350x250x8: RHSProfile
    RHS350x250x10: RHSProfile
    RHS350x250x12_5: RHSProfile
    RHS350x250x14_2: RHSProfile
    RHS350x250x16: RHSProfile
    RHS400x200x8: RHSProfile
    RHS400x200x10: RHSProfile
    RHS400x200x12_5: RHSProfile
    RHS400x200x14_2: RHSProfile
    RHS400x200x16: RHSProfile
    RHS450x250x8: RHSProfile
    RHS450x250x10: RHSProfile
    RHS450x250x12_5: RHSProfile
    RHS450x250x14_2: RHSProfile
    RHS450x250x16: RHSProfile
    RHS500x300x10: RHSProfile
    RHS500x300x12_5: RHSProfile
    RHS500x300x14_2: RHSProfile
    RHS500x300x16: RHSProfile
    RHS500x300x20: RHSProfile

class __RHSProfileParameters:
    name: str
    total_height: MM
    total_width: MM
    left_wall_thickness: MM
    right_wall_thickness: MM
    top_wall_thickness: MM
    bottom_wall_thickness: MM
    top_right_outer_radius: MM
    top_left_outer_radius: MM
    bottom_right_outer_radius: MM
    bottom_left_outer_radius: MM
    top_right_inner_radius: MM
    top_left_inner_radius: MM
    bottom_right_inner_radius: MM
    bottom_left_inner_radius: MM

RHS_PROFILES_DATABASE: dict[str, __RHSProfileParameters]
