from blueprints.structural_sections.steel.profile_definitions.rhs_profile import RHSProfile
from blueprints.type_alias import MM

class RHSCF:
    RHSCF40x20x2: RHSProfile
    RHSCF40x20x2_5: RHSProfile
    RHSCF40x20x3: RHSProfile
    RHSCF50x30x2: RHSProfile
    RHSCF50x30x2_5: RHSProfile
    RHSCF50x30x3: RHSProfile
    RHSCF50x30x4: RHSProfile
    RHSCF60x40x2: RHSProfile
    RHSCF60x40x2_5: RHSProfile
    RHSCF60x40x3: RHSProfile
    RHSCF60x40x4: RHSProfile
    RHSCF60x40x5: RHSProfile
    RHSCF70x50x2: RHSProfile
    RHSCF70x50x2_5: RHSProfile
    RHSCF70x50x3: RHSProfile
    RHSCF70x50x4: RHSProfile
    RHSCF70x50x5: RHSProfile
    RHSCF80x40x2: RHSProfile
    RHSCF80x40x2_5: RHSProfile
    RHSCF80x40x3: RHSProfile
    RHSCF80x40x4: RHSProfile
    RHSCF80x40x5: RHSProfile
    RHSCF80x60x2: RHSProfile
    RHSCF80x60x2_5: RHSProfile
    RHSCF80x60x3: RHSProfile
    RHSCF80x60x4: RHSProfile
    RHSCF80x60x5: RHSProfile
    RHSCF90x50x2: RHSProfile
    RHSCF90x50x2_5: RHSProfile
    RHSCF90x50x3: RHSProfile
    RHSCF90x50x4: RHSProfile
    RHSCF90x50x5: RHSProfile
    RHSCF100x40x2_5: RHSProfile
    RHSCF100x40x3: RHSProfile
    RHSCF100x40x4: RHSProfile
    RHSCF100x40x5: RHSProfile
    RHSCF100x50x2_5: RHSProfile
    RHSCF100x50x3: RHSProfile
    RHSCF100x50x4: RHSProfile
    RHSCF100x50x5: RHSProfile
    RHSCF100x50x6: RHSProfile
    RHSCF100x50x6_3: RHSProfile
    RHSCF100x60x2_5: RHSProfile
    RHSCF100x60x3: RHSProfile
    RHSCF100x60x4: RHSProfile
    RHSCF100x60x5: RHSProfile
    RHSCF100x60x6: RHSProfile
    RHSCF100x60x6_3: RHSProfile
    RHSCF100x80x2_5: RHSProfile
    RHSCF100x80x3: RHSProfile
    RHSCF100x80x4: RHSProfile
    RHSCF100x80x5: RHSProfile
    RHSCF100x80x6: RHSProfile
    RHSCF100x80x6_3: RHSProfile
    RHSCF120x60x2_5: RHSProfile
    RHSCF120x60x3: RHSProfile
    RHSCF120x60x4: RHSProfile
    RHSCF120x60x5: RHSProfile
    RHSCF120x60x6: RHSProfile
    RHSCF120x60x6_3: RHSProfile
    RHSCF120x60x8: RHSProfile
    RHSCF120x80x3: RHSProfile
    RHSCF120x80x4: RHSProfile
    RHSCF120x80x5: RHSProfile
    RHSCF120x80x6: RHSProfile
    RHSCF120x80x6_3: RHSProfile
    RHSCF120x80x8: RHSProfile
    RHSCF140x80x4: RHSProfile
    RHSCF140x80x5: RHSProfile
    RHSCF140x80x6: RHSProfile
    RHSCF140x80x6_3: RHSProfile
    RHSCF140x80x8: RHSProfile
    RHSCF150x100x4: RHSProfile
    RHSCF150x100x5: RHSProfile
    RHSCF150x100x6: RHSProfile
    RHSCF150x100x6_3: RHSProfile
    RHSCF150x100x8: RHSProfile
    RHSCF150x100x10: RHSProfile
    RHSCF150x100x12: RHSProfile
    RHSCF150x100x12_5: RHSProfile
    RHSCF160x80x4: RHSProfile
    RHSCF160x80x5: RHSProfile
    RHSCF160x80x6: RHSProfile
    RHSCF160x80x6_3: RHSProfile
    RHSCF160x80x8: RHSProfile
    RHSCF160x80x10: RHSProfile
    RHSCF160x80x12: RHSProfile
    RHSCF160x80x12_5: RHSProfile
    RHSCF180x100x4: RHSProfile
    RHSCF180x100x5: RHSProfile
    RHSCF180x100x6: RHSProfile
    RHSCF180x100x6_3: RHSProfile
    RHSCF180x100x8: RHSProfile
    RHSCF180x100x10: RHSProfile
    RHSCF180x100x12: RHSProfile
    RHSCF180x100x12_5: RHSProfile
    RHSCF200x100x4: RHSProfile
    RHSCF200x100x5: RHSProfile
    RHSCF200x100x6: RHSProfile
    RHSCF200x100x6_3: RHSProfile
    RHSCF200x100x8: RHSProfile
    RHSCF200x100x10: RHSProfile
    RHSCF200x100x12: RHSProfile
    RHSCF200x100x12_5: RHSProfile
    RHSCF200x120x4: RHSProfile
    RHSCF200x120x5: RHSProfile
    RHSCF200x120x6: RHSProfile
    RHSCF200x120x6_3: RHSProfile
    RHSCF200x120x8: RHSProfile
    RHSCF200x120x10: RHSProfile
    RHSCF200x120x12: RHSProfile
    RHSCF200x120x12_5: RHSProfile
    RHSCF250x150x5: RHSProfile
    RHSCF250x150x6: RHSProfile
    RHSCF250x150x6_3: RHSProfile
    RHSCF250x150x8: RHSProfile
    RHSCF250x150x10: RHSProfile
    RHSCF250x150x12: RHSProfile
    RHSCF250x150x12_5: RHSProfile
    RHSCF250x150x16: RHSProfile
    RHSCF260x180x5: RHSProfile
    RHSCF260x180x6_3: RHSProfile
    RHSCF260x180x8: RHSProfile
    RHSCF260x180x10: RHSProfile
    RHSCF260x180x12: RHSProfile
    RHSCF260x180x12_5: RHSProfile
    RHSCF260x180x16: RHSProfile
    RHSCF300x100x6: RHSProfile
    RHSCF300x100x6_3: RHSProfile
    RHSCF300x100x8: RHSProfile
    RHSCF300x100x10: RHSProfile
    RHSCF300x100x12: RHSProfile
    RHSCF300x100x12_5: RHSProfile
    RHSCF300x100x16: RHSProfile
    RHSCF300x150x6: RHSProfile
    RHSCF300x150x6_3: RHSProfile
    RHSCF300x150x8: RHSProfile
    RHSCF300x150x10: RHSProfile
    RHSCF300x150x12: RHSProfile
    RHSCF300x150x12_5: RHSProfile
    RHSCF300x150x16: RHSProfile
    RHSCF300x200x6: RHSProfile
    RHSCF300x200x6_3: RHSProfile
    RHSCF300x200x8: RHSProfile
    RHSCF300x200x10: RHSProfile
    RHSCF300x200x12: RHSProfile
    RHSCF300x200x12_5: RHSProfile
    RHSCF300x200x16: RHSProfile
    RHSCF350x250x6: RHSProfile
    RHSCF350x250x6_3: RHSProfile
    RHSCF350x250x8: RHSProfile
    RHSCF350x250x10: RHSProfile
    RHSCF350x250x12: RHSProfile
    RHSCF350x250x12_5: RHSProfile
    RHSCF350x250x16: RHSProfile
    RHSCF400x200x8: RHSProfile
    RHSCF400x200x12_5: RHSProfile
    RHSCF400x200x16: RHSProfile
    RHSCF400x300x8: RHSProfile
    RHSCF400x300x10: RHSProfile
    RHSCF400x300x12: RHSProfile
    RHSCF400x300x12_5: RHSProfile
    RHSCF400x300x16: RHSProfile

class __RHSCFProfileParameters:
    name: str
    total_height: MM
    total_width: MM
    left_wall_thickness: MM
    right_wall_thickness: MM
    top_wall_thickness: MM
    bottom_wall_thickness: MM
    top_right_outer_radius: MM
    top_left_outer_radius: MM
    bottom_right_outer_radius: MM
    bottom_left_outer_radius: MM
    top_right_inner_radius: MM
    top_left_inner_radius: MM
    bottom_right_inner_radius: MM
    bottom_left_inner_radius: MM

RHSCF_PROFILES_DATABASE: dict[str, __RHSCFProfileParameters]
