"""Fixtures for steel strength check tests."""

import pytest

from blueprints.materials.steel import SteelMaterial, SteelStrengthClass
from blueprints.structural_sections.steel.standard_profiles.chs import CHS
from blueprints.structural_sections.steel.standard_profiles.heb import HEB
from blueprints.structural_sections.steel.steel_cross_section import SteelCrossSection


@pytest.fixture(scope="class")
def heb_steel_cross_section() -> SteelCrossSection:
    """Create a SteelCrossSection fixture with HEB300 profile and S355 steel material."""
    steel_material = SteelMaterial(steel_class=SteelStrengthClass.S355)
    profile = HEB.HEB300
    return SteelCrossSection(profile=profile, material=steel_material)


@pytest.fixture(scope="class")
def chs_steel_cross_section() -> SteelCrossSection:
    """Create a SteelCrossSection fixture with CHS profile and S355 steel material."""
    steel_material = SteelMaterial(steel_class=SteelStrengthClass.S355)
    profile = CHS.CHS1016x12_5
    return SteelCrossSection(profile=profile, material=steel_material)
