from blueprints.structural_sections.steel.profile_definitions.strip_profile import StripProfile
from blueprints.type_alias import MM

class Strip:
    STRIP160x5: StripProfile
    STRIP160x6: StripProfile
    STRIP160x8: StripProfile
    STRIP160x10: StripProfile
    STRIP160x12: StripProfile
    STRIP160x15: StripProfile
    STRIP160x20: StripProfile
    STRIP160x25: StripProfile
    STRIP160x30: StripProfile
    STRIP165x5: StripProfile
    STRIP165x6: StripProfile
    STRIP165x8: StripProfile
    STRIP165x10: StripProfile
    STRIP165x12: StripProfile
    STRIP165x15: StripProfile
    STRIP165x20: StripProfile
    STRIP165x25: StripProfile
    STRIP165x30: StripProfile
    STRIP170x5: StripProfile
    STRIP170x6: StripProfile
    STRIP170x8: StripProfile
    STRIP170x10: StripProfile
    STRIP170x12: StripProfile
    STRIP170x15: StripProfile
    STRIP170x20: StripProfile
    STRIP180x5: StripProfile
    STRIP180x6: StripProfile
    STRIP180x8: StripProfile
    STRIP180x10: StripProfile
    STRIP180x12: StripProfile
    STRIP180x15: StripProfile
    STRIP180x20: StripProfile
    STRIP180x25: StripProfile
    STRIP180x30: StripProfile
    STRIP180x40: StripProfile
    STRIP180x50: StripProfile
    STRIP200x5: StripProfile
    STRIP200x6: StripProfile
    STRIP200x8: StripProfile
    STRIP200x10: StripProfile
    STRIP200x12: StripProfile
    STRIP200x15: StripProfile
    STRIP200x20: StripProfile
    STRIP200x25: StripProfile
    STRIP200x30: StripProfile
    STRIP200x40: StripProfile
    STRIP200x50: StripProfile
    STRIP220x6: StripProfile
    STRIP220x8: StripProfile
    STRIP220x10: StripProfile
    STRIP220x12: StripProfile
    STRIP220x15: StripProfile
    STRIP220x20: StripProfile
    STRIP220x25: StripProfile
    STRIP220x30: StripProfile
    STRIP230x6: StripProfile
    STRIP230x8: StripProfile
    STRIP230x10: StripProfile
    STRIP230x12: StripProfile
    STRIP230x15: StripProfile
    STRIP230x20: StripProfile
    STRIP230x25: StripProfile

class __StripProfileParameters:
    name: str
    width: MM
    height: MM

STRIP_PROFILES_DATABASE: dict[str, __StripProfileParameters]
