from blueprints.structural_sections.steel.profile_definitions.rhs_profile import RHSProfile
from blueprints.type_alias import MM

class SHS:
    SHS40x2_6: RHSProfile
    SHS40x3_2: RHSProfile
    SHS40x4: RHSProfile
    SHS40x5: RHSProfile
    SHS50x2_6: RHSProfile
    SHS50x3_2: RHSProfile
    SHS50x4: RHSProfile
    SHS50x5: RHSProfile
    SHS50x6_3: RHSProfile
    SHS60x2_6: RHSProfile
    SHS60x3_2: RHSProfile
    SHS60x4: RHSProfile
    SHS60x5: RHSProfile
    SHS60x6_3: RHSProfile
    SHS60x8: RHSProfile
    SHS70x3_2: RHSProfile
    SHS70x4: RHSProfile
    SHS70x5: RHSProfile
    SHS70x6_3: RHSProfile
    SHS70x8: RHSProfile
    SHS80x3_2: RHSProfile
    SHS80x4: RHSProfile
    SHS80x5: RHSProfile
    SHS80x6_3: RHSProfile
    SHS80x8: RHSProfile
    SHS90x4: RHSProfile
    SHS90x5: RHSProfile
    SHS90x6_3: RHSProfile
    SHS90x8: RHSProfile
    SHS100x4: RHSProfile
    SHS100x5: RHSProfile
    SHS100x6_3: RHSProfile
    SHS100x8: RHSProfile
    SHS100x10: RHSProfile
    SHS120x5: RHSProfile
    SHS120x6_3: RHSProfile
    SHS120x8: RHSProfile
    SHS120x10: RHSProfile
    SHS120x12_5: RHSProfile
    SHS140x5: RHSProfile
    SHS140x6_3: RHSProfile
    SHS140x8: RHSProfile
    SHS140x10: RHSProfile
    SHS140x12_5: RHSProfile
    SHS150x5: RHSProfile
    SHS150x6_3: RHSProfile
    SHS150x8: RHSProfile
    SHS150x10: RHSProfile
    SHS150x12_5: RHSProfile
    SHS150x14_2: RHSProfile
    SHS150x16: RHSProfile
    SHS160x5: RHSProfile
    SHS160x6_3: RHSProfile
    SHS160x8: RHSProfile
    SHS160x10: RHSProfile
    SHS160x12_5: RHSProfile
    SHS160x14_2: RHSProfile
    SHS160x16: RHSProfile
    SHS180x5: RHSProfile
    SHS180x6_3: RHSProfile
    SHS180x8: RHSProfile
    SHS180x10: RHSProfile
    SHS180x12_5: RHSProfile
    SHS180x14_2: RHSProfile
    SHS180x16: RHSProfile
    SHS200x5: RHSProfile
    SHS200x6_3: RHSProfile
    SHS200x8: RHSProfile
    SHS200x10: RHSProfile
    SHS200x12_5: RHSProfile
    SHS200x14_2: RHSProfile
    SHS200x16: RHSProfile
    SHS220x6_3: RHSProfile
    SHS220x8: RHSProfile
    SHS220x10: RHSProfile
    SHS220x12_5: RHSProfile
    SHS220x14_2: RHSProfile
    SHS220x16: RHSProfile
    SHS250x6_3: RHSProfile
    SHS250x8: RHSProfile
    SHS250x10: RHSProfile
    SHS250x12_5: RHSProfile
    SHS250x14_2: RHSProfile
    SHS250x16: RHSProfile
    SHS260x6_3: RHSProfile
    SHS260x8: RHSProfile
    SHS260x10: RHSProfile
    SHS260x12_5: RHSProfile
    SHS260x14_2: RHSProfile
    SHS260x16: RHSProfile
    SHS300x6_3: RHSProfile
    SHS300x8: RHSProfile
    SHS300x10: RHSProfile
    SHS300x12_5: RHSProfile
    SHS300x14_2: RHSProfile
    SHS300x16: RHSProfile
    SHS350x8: RHSProfile
    SHS350x10: RHSProfile
    SHS350x12_5: RHSProfile
    SHS350x14_2: RHSProfile
    SHS350x16: RHSProfile
    SHS400x10: RHSProfile
    SHS400x12_5: RHSProfile
    SHS400x14_2: RHSProfile
    SHS400x16: RHSProfile
    SHS400x20: RHSProfile

class __SHSProfileParameters:
    name: str
    total_height: MM
    total_width: MM
    left_wall_thickness: MM
    right_wall_thickness: MM
    top_wall_thickness: MM
    bottom_wall_thickness: MM
    top_right_outer_radius: MM
    top_left_outer_radius: MM
    bottom_right_outer_radius: MM
    bottom_left_outer_radius: MM
    top_right_inner_radius: MM
    top_left_inner_radius: MM
    bottom_right_inner_radius: MM
    bottom_left_inner_radius: MM

SHS_PROFILES_DATABASE: dict[str, __SHSProfileParameters]
